CON

  _CLKMODE            = XTAL1 + PLL16X
  _XINFREQ            = 5_000_000

  'Clock Frequency Equation
  CLK_FREQ            = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ

  'X Axis Pin Assignments  
  X_DIRECTION         = 22
  X_STEP              = 23
  X_DISABLE           = 24
  
  'Y Axis Pin Assignments
  Y_DISABLE           = 25
  Y_STEP              = 26  
  Y_DIRECTION         = 27

  'X Axis Driver Constants
  X_PULSE_WIDTH       = CLK_FREQ / 1_000_000 '= 80
  X_MIN_SPEED         = CLK_FREQ / 5_714 '= 14_000 Rounded
  X_MAX_SPEED         = CLK_FREQ / 16_000 '= 5_000
  X_RAMP_INC_DEC      = CLK_FREQ / 26_666_666 '= 3 Rounded
  X_INC_DIR           = 0 '= Counter-clockwise rotation
  X_DEC_DIR           = 1 '= Clockwise rotation
  X_STEPS_PER_REV     = 2_000 '= 200 Full steps * 10 microsteps
  X_STEPS_PER_METER   = 55_555  

  'X Axis Stage Constants
  X_MAX_STEPS         = 16_933 '12 inches of linear travel

  'Y Axis Driver Constants
  Y_PULSE_WIDTH       = CLK_FREQ / 1_000_000 '= 80
  Y_MIN_SPEED         = CLK_FREQ / 5_714 '= 14_000 Rounded
  Y_MAX_SPEED         = CLK_FREQ / 16_000 '= 5_000
  Y_RAMP_INC_DEC      = CLK_FREQ / 26_666_666 '= 3 Rounded
  Y_INC_DIR           = 1 '= Counter-clockwise rotation
  Y_DEC_DIR           = 0 '= Clockwise rotation
  Y_STEPS_PER_REV     = 2_000 '= 200 Full steps * 10 microsteps
  Y_STEPS_PER_METER   = 55_555

  'Y Axis Stage Constants
  Y_MAX_STEPS         = 16_933 '12 inches of linear travel

  'Pin utilized for syncronizing the stages
  STAGE_SYNCRONIZER         = 18

OBJ

  X:            "StepperDriver"
  Y:            "StepperDriver"

PUB Main

  'Initialize the stepper driver for the X axis
  X.Start(X_DIRECTION, X_STEP, X_DISABLE, X_INC_DIR, X_DEC_DIR, X_PULSE_WIDTH, X_MAX_SPEED, X_MIN_SPEED, X_RAMP_INC_DEC, X_STEPS_PER_REV)

  'Initialize the stepper driver for the Y axis
  Y.Start(Y_DIRECTION, Y_STEP, Y_DISABLE, Y_INC_DIR, Y_DEC_DIR, Y_PULSE_WIDTH, Y_MAX_SPEED, Y_MIN_SPEED, Y_RAMP_INC_DEC, Y_STEPS_PER_REV)

  WAITCNT(CNT + (3 * CLKFREQ))

  DIRA[STAGE_SYNCRONIZER]~~
  OUTA[STAGE_SYNCRONIZER]~

  TestStageSyncronization

PUB TestStageSyncronization | XRampingSteps, XRunningSteps, YRampingSteps, YRunningSteps

  'Ramping and running steps can and should be computed outside of the child object,
  'but I am currently just leaving them in there as a matter of convenience
  XRampingSteps := X.GetRampingSteps(X_MAX_STEPS)
  XRunningSteps := X.GetRunningSteps(X_MAX_STEPS)

  YRampingSteps := Y.GetRampingSteps(Y_MAX_STEPS)
  YRunningSteps := Y.GetRunningSteps(Y_MAX_STEPS)
  '////////////////////////////////////////////////////////////////////////////////

  X.MoveStagePosition(X_INC_DIR, XRampingSteps, XRunningSteps, STAGE_SYNCRONIZER)
  Y.MoveStagePosition(Y_INC_DIR, YRampingSteps, YRunningSteps, STAGE_SYNCRONIZER)

  OUTA[STAGE_SYNCRONIZER]~~

  REPEAT WHILE X.IsMoveComplete == FALSE AND Y.IsMoveComplete == FALSE

  OUTA[STAGE_SYNCRONIZER]~

  WAITCNT(CNT + (5 * CLKFREQ))

  X.MoveStagePosition(X_DEC_DIR, XRampingSteps, XRunningSteps, STAGE_SYNCRONIZER)
  Y.MoveStagePosition(Y_DEC_DIR, YRampingSteps, YRunningSteps, STAGE_SYNCRONIZER)

  OUTA[STAGE_SYNCRONIZER]~~

  X.Stop
  Y.Stop
